﻿using System;
using System.Data;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NAudio.Wave;
using Capstone;

namespace Capstone.Project
{
    class CustomSoundClass
    {
        //This is the default format to be used
        public static WaveFormat DefaultFormat = new WaveFormat(44100, 2);

        /*
         * This function handles the concatentation of wave files.
         * Making sure that they are all the same format and outputing them to the specified path.
         */ 
        public static void ConcatenateWavFiles(string outputFile, Queue<dataBaseRow> wordQueue)
        {                
            //this through try catch was taken from online
            //http://stackoverflow.com/questions/6777340/how-to-join-2-or-more-wav-files-together-programatically
            byte[] buffer = new byte[1024];
            WaveFileWriter waveFileWriter = null;

            try
            {
                while (wordQueue.Count > 0)
                {
                    dataBaseRow soundFile = wordQueue.Dequeue();
                    using (WaveFileReader reader = new WaveFileReader(soundFile.fileLocation))
                    {
                        if (waveFileWriter == null)
                        {
                            // first time in create new Writer
                            waveFileWriter = new WaveFileWriter(outputFile, DefaultFormat);
                        }
                        else
                        {
                            if (!reader.WaveFormat.Equals(waveFileWriter.WaveFormat))
                            {
                                //throw new InvalidOperationException("Can't concatenate WAV Files that don't share the same format");

                                WaveStream stream = new WaveFileReader(soundFile.fileLocation);
                                WaveFormat testFormat = stream.WaveFormat;
                                stream.Close();

                                //converter
                                if (testFormat.Equals(CustomSoundClass.DefaultFormat) == false)
                                {
                                    if (File.Exists("origFile.wav"))
                                    {
                                        File.Delete("origFile.wav");
                                    }
                                    File.Move(soundFile.fileLocation, "origFile.wav");

                                    stream = new WaveFileReader("origFile.wav");

                                    WaveFormatConversionStream str = new WaveFormatConversionStream(CustomSoundClass.DefaultFormat, stream);

                                    WaveFileWriter.CreateWaveFile(soundFile.fileLocation, str);

                                    stream.Close();
                                    str.Close();

                                    File.Delete("origFile.wav");
                                }
                            }
                        }

                        int read;
                        while ((read = reader.Read(buffer, 0, buffer.Length)) > 0)
                        {
                            waveFileWriter.WriteData(buffer, 0, read);
                        }
                    }
                }
            }
            finally
            {
                if (waveFileWriter != null)
                {
                    waveFileWriter.Dispose();
                }
            }
        }

        //http://stackoverflow.com/questions/6465863/trim-an-audio-file-wav-mp3
        public static void TrimWavFile(string inPath, string outPath, TimeSpan cutFromStart, TimeSpan cutFromEnd)
        {
            using (WaveFileReader reader = new WaveFileReader(inPath))
            {
                using (WaveFileWriter writer = new WaveFileWriter(outPath, reader.WaveFormat))
                {
                    int bytesPerMillisecond = reader.WaveFormat.AverageBytesPerSecond / 1000;

                    int startPos = (int)cutFromStart.TotalMilliseconds * bytesPerMillisecond;
                    startPos = startPos - startPos % reader.WaveFormat.BlockAlign;

                    int endPos = (int)cutFromEnd.TotalMilliseconds * bytesPerMillisecond;
                    endPos = endPos - endPos % reader.WaveFormat.BlockAlign;
                    //int endPos = (int)reader.Length - endBytes;

                    TrimWavFile(reader, writer, startPos, endPos);
                }
            }
        }
        
        //http://stackoverflow.com/questions/6465863/trim-an-audio-file-wav-mp3
        private static void TrimWavFile(WaveFileReader reader, WaveFileWriter writer, int startPos, int endPos)
        {
            reader.Position = startPos;
            byte[] buffer = new byte[1024];
            while (reader.Position < endPos)
            {
                int bytesRequired = (int)(endPos - reader.Position);
                if (bytesRequired > 0)
                {
                    int bytesToRead = Math.Min(bytesRequired, buffer.Length);
                    int bytesRead = reader.Read(buffer, 0, bytesToRead);
                    if (bytesRead > 0)
                    {
                        writer.WriteData(buffer, 0, bytesRead);
                    }
                }
            }
            writer.Close();
        }
    }
}
