﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using NAudio.Wave;

namespace Capstone.Project
{
    public partial class snippetCreatorForm : Form
    {
        //use a constant for the clip that has been loaded
        private const string loadedClip = "loadedClip.wav";

        //playback variables
        private DirectSoundOut audioOutput = new DirectSoundOut();
        private WaveFileReader wfr;
        WaveChannel32 wc;

        //first mark variables
        private Boolean firstMarked = false;
        private TimeSpan firstMark;
        private Point topPointStart;
        private Point bottomPointStart;

        //second mark variables
        private Boolean secondMarked = false;
        private TimeSpan secondMark;
        private Point topPointEnd;
        private Point bottomPointEnd;

        //database connections
        private OleDbConnection theConnection;
        private string theConnectionString;
        private String filePath = @"SoundClipCollection\";
        
        private Pen gridPen = new Pen(Color.Black, 1);

        private bool playing = false;
        private bool fileLoaded = false;

        /*
         * Default constructor for the form
         */
        public snippetCreatorForm()
        {
            InitializeComponent();
        }

        /*
         * This constructor is used when a file is being loaded into the form
         */
        public snippetCreatorForm(string newFile)
        {
            InitializeComponent();

            //load the new file
            if (File.Exists(loadedClip))
            {
                //delete any old files that were left behind
                File.Delete(loadedClip);
                //copy the new file
                File.Copy(newFile, loadedClip);
            }
            else
            {
                //copy the new file
                File.Copy(newFile, loadedClip);
            }
            //'open' the new file
            openNewFile(loadedClip);

            //set button enabled states
            toolStripButtonPlay.Enabled = true;
            toolStripButtonResumePlaying.Enabled = true;
            toolStripButtonMarkStart.Enabled = true;
        }        

        private void snippetCreater_FormClosing(object sender, FormClosingEventArgs e)
        {
            //stop audio output and dispose
            audioOutput.Stop();
            audioOutput.Dispose();

            //close the wave channel
            if (wc != null)
            {
                wc.Close();
            }

            //close the wave file reader
            if (wfr != null)
            {
                wfr.Close();
            }

            //delete the temporary file
            if (File.Exists(loadedClip))
            {
                File.Delete(loadedClip);
            }
        }        
        
        private void toolStripButtonPlay_Click(object sender, EventArgs e)
        {
            playing = true;

            toolStripButtonPause.Enabled = true;

            toolStripButtonPlay.Enabled = false;
            toolStripButtonResumePlaying.Enabled = false;
            toolStripButtonMarkStart.Enabled = false;
            toolStripButtonMarkEnd.Enabled = false;
            toolStripButtonMovetoStart.Enabled = false;
            timerPreviewSnippet.Enabled = false;
            toolStripButtonClearMarks.Enabled = false;
            toolStripButtonSaveSnippet.Enabled = false;

            //set the current time to the beggining
            wc.CurrentTime = new TimeSpan(0);
            moveTrackBar(wc.CurrentTime, wc.TotalTime);
            
            //start the playback timer
            timerCheckPlayback.Start();
            
            //start playback
            audioOutput.Play();            
        }

        /***********************************************
         *Starts playing from the current location
         ***********************************************/
        private void toolStripButtonPlayFromCurrentLocation_Click(object sender, EventArgs e)
        {
            playing = true;

            toolStripButtonPause.Enabled = true;

            toolStripButtonPlay.Enabled = false;
            toolStripButtonResumePlaying.Enabled = false;
            toolStripButtonMarkStart.Enabled = false;
            toolStripButtonMarkEnd.Enabled = false;
            toolStripButtonMovetoStart.Enabled = false;
            timerPreviewSnippet.Enabled = false;
            toolStripButtonClearMarks.Enabled = false;
            toolStripButtonSaveSnippet.Enabled = false;

            timerCheckPlayback.Start();

            audioOutput.Play();
        }

        /***********************************************
         *Pause playback
         **********************************************/
        private void toolStripButtonPause_Click(object sender, EventArgs e)
        {
            //pause audio playback
            audioOutput.Pause();
            //stop the timer
            timerCheckPlayback.Stop();

            playing = false;

            //move the trackbar
            moveTrackBar(wc.CurrentTime, wc.TotalTime);

            toolStripButtonMarkStart.Enabled = true;
            toolStripButtonPlay.Enabled = true;
            toolStripButtonResumePlaying.Enabled = true;

            if(firstMarked == true)
            {
                toolStripButtonMovetoStart.Enabled = true;
                toolStripButtonMarkEnd.Enabled = true;
            }

            if(firstMarked && secondMarked)
            {
                toolStripButtonPreviewSnippet.Enabled = true;
                toolStripButtonClearMarks.Enabled = true;
                toolStripButtonSaveSnippet.Enabled = true;
            }

            toolStripButtonPause.Enabled = false;

            if (firstMarked == true)
            {
                toolStripButtonMarkEnd.Enabled = true;
            }
        }

        /***********************************************
         *Mark the start of the snippet
         **********************************************/
        private void toolStripButtonMarkStart_Click(object sender, EventArgs e)
        {
            //add a start line on the track bar.

            if (secondMarked == true)
            {
                secondMarked = false;
                toolStripButtonSaveSnippet.Enabled = false;
                toolStripButtonPreviewSnippet.Enabled = false;
            }

            firstMarked = true;

            double percentage = wc.CurrentTime.TotalMilliseconds / wc.TotalTime.TotalMilliseconds;

            firstMark = wc.CurrentTime;
            topPointStart = new Point((int)(percentage * panelSpacerMiddle.Width), 0);
            bottomPointStart = new Point((int)(percentage * panelSpacerMiddle.Width), panelSpacer.Height);
            
            //force a repaint
            panelSpacerMiddle.Invalidate();
            panelSpacerMiddle.Update();    

            toolStripButtonMovetoStart.Enabled = true;
            toolStripButtonMarkEnd.Enabled = true;            
        }

        /***********************************************
         *Mark the end of the snippet
         **********************************************/
        private void toolStripButtonMarkEnd_Click(object sender, EventArgs e)
        {
            if (wc.CurrentTime > firstMark && firstMarked == true)
            {
                secondMarked = true;
                secondMark = wc.CurrentTime;

                double percentage = wc.CurrentTime.TotalMilliseconds / wc.TotalTime.TotalMilliseconds;
                topPointEnd = new Point((int)(percentage * panelSpacerMiddle.Width), 0);
                bottomPointEnd = new Point((int)(percentage * panelSpacerMiddle.Width), panelSpacer.Height);
                
                //force a repaint
                panelSpacerMiddle.Invalidate();
                panelSpacerMiddle.Update();

                toolStripButtonPreviewSnippet.Enabled = true;
                toolStripButtonClearMarks.Enabled = true;
                toolStripButtonSaveSnippet.Enabled = true;
            }
            else
            {
                MessageBox.Show("Second mark must be made after the first mark.");
            }            
        }

        /***********************************************
         *Clear both snippet markers
         **********************************************/
        private void toolStripButtonClearMark_Click(object sender, EventArgs e)
        {
            toolStripButtonMarkEnd.Enabled = false;
            toolStripButtonMovetoStart.Enabled = false;
            toolStripButtonPreviewSnippet.Enabled = false;
            toolStripButtonSaveSnippet.Enabled = false;

            toolStripButtonMarkStart.Enabled = true;

            firstMarked = false;
            secondMarked = false;

            //force a repaint
            Invalidate();
            Refresh();
        }

        /***********************************************
         * handles the moving of the tack bar
         **********************************************/
        private void moveTrackBar(TimeSpan start, TimeSpan max)
        {
            double percentage = start.TotalMilliseconds / max.TotalMilliseconds;
            int newValue = (int)(percentage * trackBarPlaybackPosition.Maximum);
            trackBarPlaybackPosition.Value = newValue;
        }

        /*********************************************************************************
         *Previews the loaded file starting at the first mark
         *********************************************************************************/
        private void toolStripButtonPreviewMark_Click(object sender, EventArgs e)
        {
            if (firstMarked == true)
            {
                wc.CurrentTime = firstMark;
                moveTrackBar(firstMark, wc.TotalTime);
            }
            else 
            {
                MessageBox.Show("First mark nees to be made");
            }            
        }

        /*******************************************************************
         *Handles opening a new file and initializing the required variables
         *******************************************************************/
        private void openNewFile(string fileName)
        {
            wfr = new WaveFileReader(fileName);
            wc = new WaveChannel32(wfr);

            
            wc.PadWithZeroes = false;
            audioOutput.Init(wc);

            trackBarPlaybackPosition.Maximum = (int)wc.TotalTime.TotalMilliseconds;


            fileLoaded = true;
        }

        /*******************************************************************
         *Handles changing the current time when the trackbar values changes
         *******************************************************************/
        private void trackBarPlaybackPosition_ValueChanged(object sender, EventArgs e)
        {
            if(wc != null)
            {
                wc.CurrentTime = new TimeSpan(0, 0, 0, 0, trackBarPlaybackPosition.Value);
            }
        }

        /***********************************************
         *Save the snippet that has been selected
         *Makes sure that the first and second marks have been selected,
         *the tag has been filled out,
         *and that the person has been selected
         **********************************************/
        private void toolStripButtonSaveSnippet_Click(object sender, EventArgs e)
        {
            if ((firstMarked== true && secondMarked == true) && (firstMark < secondMark) && textBoxSnippetTag.Text.Trim(' ') != "" && comboBoxPeople.SelectedIndex > 0)
            {
                Console.WriteLine("Inside the if statement.");

                //get rid of all connections
                //stop audio output and dispose
                audioOutput.Stop();
                audioOutput.Dispose();

                TimeSpan currentLocation = wc.CurrentTime;

                //close the wave channel
                if (wc != null)
                {
                    wc.Close();
                }

                //close the wave file reader
                if (wfr != null)
                {
                    wfr.Close();
                }

                CustomSoundClass.TrimWavFile(loadedClip, "temp.wav", firstMark, secondMark);
                
                string snippetTag = textBoxSnippetTag.Text.ToLower();
                int hash = snippetTag.GetHashCode();
                string person = comboBoxPeople.SelectedItem.ToString();
                string fileLocation = filePath + person + "\\" + snippetTag + ".wav";

                theConnectionString = "Provider=Microsoft.ACE.OLEDB.12.0;"
                           + @"data source=soundClips.accdb";   // Access 2010
                theConnection = new OleDbConnection(theConnectionString);
                theConnection.Open();

                OleDbCommand searchCommand;
                searchCommand = theConnection.CreateCommand();

                searchCommand.CommandText = String.Format("SELECT * FROM SoundClips WHERE hash = {0} AND word = '{1}' AND person = '{2}'", hash, snippetTag, person);
                OleDbDataReader reader = searchCommand.ExecuteReader();

                // used for part 4.b
                if (reader.HasRows == false)
                {
                    Console.WriteLine("Word is not in the database.");
                    OleDbCommand insertCommand;
                    insertCommand = theConnection.CreateCommand();
                    //ID(auto gen), hash-int, word-string, fileLocation-str, person-string
                    insertCommand.CommandText = String.Format("INSERT INTO SoundClips VALUES ({0},'{1}','{2}', '{3}')", hash, snippetTag, fileLocation, person);
                    insertCommand.ExecuteNonQuery();

                    wfr.Close();

                    Int32 index = fileLocation.IndexOf(snippetTag + ".wav");

                    Directory.CreateDirectory(fileLocation.Remove(index));

                    WaveStream stream = new WaveFileReader("temp.wav");
                    WaveFormat testFormat = stream.WaveFormat;
                    stream.Close();

                    //converter
                    if (testFormat.Equals(CustomSoundClass.DefaultFormat) == false)
                    {
                        if (File.Exists("origFile.wav"))
                        {
                            File.Delete("origFile.wav");
                        }
                        File.Move("temp.wav", "origFile.wav");

                        stream = new WaveFileReader("origFile.wav");

                        WaveFormatConversionStream str = new WaveFormatConversionStream(CustomSoundClass.DefaultFormat, stream);

                        WaveFileWriter.CreateWaveFile("temp.wav", str);

                        stream.Close();
                        str.Close();

                        File.Delete("origFile.wav");
                    }

                    File.Move("temp.wav", fileLocation);

                    MessageBox.Show("snippet successfully created");
                }
                else
                {
                    MessageBox.Show("Word has already been added to the database.");
                    File.Delete("temp.wav");
                }

                theConnection.Close();

                openNewFile(loadedClip);

                wc.CurrentTime = currentLocation;
            }
            else
            {
                MessageBox.Show("Please enter a tag for the snippet or select a person to assign this snippet to");
            }
        }

        /***********************************************
         *Stop playback
         **********************************************/
        private void toolStripButtonStop_Click(object sender, EventArgs e)
        {
            audioOutput.Pause();
            timerCheckPlayback.Stop();

            //move the trackbar
            wc.CurrentTime = new TimeSpan(0);
            moveTrackBar(wc.CurrentTime, wc.TotalTime);
            toolStripButtonPlay.Enabled = true;
            toolStripButtonPause.Enabled = false;
        }

        /***********************************************
         *Paint method for the spacer
         **********************************************/
        private void panelSpacerMiddle_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                e.Graphics.Clear(this.BackColor);
                //repaint the graphics    
                if (firstMarked == true)
                {
                    e.Graphics.DrawLine(gridPen, topPointStart, bottomPointStart);
                }

                if (secondMarked == true)
                {
                    e.Graphics.DrawLine(gridPen, topPointEnd, bottomPointEnd);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + Environment.NewLine + "In GameForm_Paint");
            }
        }

        /***********************************************
         *Starts playing from the current location and stop playing at the second mark
         **********************************************/
        private void toolStripButtonPreviewSnippet_Click(object sender, EventArgs e)
        {
            if(firstMarked == true && secondMarked == true)
            {
                playing = true;

                toolStripButtonPause.Enabled = true;

                toolStripButtonPlay.Enabled = false;
                toolStripButtonResumePlaying.Enabled = false;
                toolStripButtonMarkStart.Enabled = false;
                toolStripButtonMarkEnd.Enabled = false;
                toolStripButtonMovetoStart.Enabled = false;
                timerPreviewSnippet.Enabled = false;
                toolStripButtonClearMarks.Enabled = false;
                toolStripButtonSaveSnippet.Enabled = false;

                wc.CurrentTime = firstMark;
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
                timerPreviewSnippet.Start();
                audioOutput.Play();                
            }
            else
            {
                MessageBox.Show("The start and end of a snippet is required.");
            }
        }

        /***********************************************
         *Timer which is used to determine when to stop playback.
         *Will stop at the end of the file
         **********************************************/
        private void timerCheckPlayback_Tick(object sender, EventArgs e)
        {
            moveTrackBar(wc.CurrentTime, wc.TotalTime);

            if (audioOutput.PlaybackState == PlaybackState.Stopped)
            {
                //just stop the playback timer
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
                audioOutput.Stop();                

                toolStripButtonPlay.Enabled = true;
                toolStripButtonResumePlaying.Enabled = true;
                toolStripButtonPause.Enabled = false;
                toolStripButtonMarkStart.Enabled = true;
                toolStripButtonMovetoStart.Enabled = true;
                toolStripButtonMarkEnd.Enabled = true;
                toolStripButtonClearMarks.Enabled = true;

                if (secondMarked == true)
                {
                    toolStripButtonPreviewSnippet.Enabled = true;
                    toolStripButtonSaveSnippet.Enabled = true;
                }                

                timerCheckPlayback.Stop();
            }
        }

        /***********************************************
         *Timer that handles playing back just the selected snippet
         **********************************************/
        private void timerPreviewSnippet_Tick(object sender, EventArgs e)
        {
            moveTrackBar(wc.CurrentTime, wc.TotalTime);

            if (audioOutput.PlaybackState == PlaybackState.Stopped || wc.CurrentTime >= secondMark)
            {
                //just stop the playback timer
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
                audioOutput.Stop();

                toolStripButtonPlay.Enabled = true;
                toolStripButtonResumePlaying.Enabled = true;
                toolStripButtonPause.Enabled = false;
                toolStripButtonMarkStart.Enabled = true;
                toolStripButtonMovetoStart.Enabled = true;
                toolStripButtonMarkEnd.Enabled = true;
                toolStripButtonPreviewSnippet.Enabled = true;
                toolStripButtonClearMarks.Enabled = true;
                toolStripButtonSaveSnippet.Enabled = true;


                timerPreviewSnippet.Stop();
            }
        }

        /***********************************************
         *Handles all of the possible keydowns
         **********************************************/
        private void snippetCreatorForm_KeyDown(object sender, KeyEventArgs e)
        {
            bool skipSpacePress = true;
            if (e.KeyValue == 32 && fileLoaded == true && skipSpacePress == false)
            {
                //do something
                if(playing == true && wc.CurrentTime != wc.TotalTime)
                {
                    playing = false;
                    audioOutput.Pause();
                    timerCheckPlayback.Stop();
                    timerPreviewSnippet.Stop();

                    //move the trackbar
                    moveTrackBar(wc.CurrentTime, wc.TotalTime);

                    toolStripButtonMarkStart.Enabled = true;
                    toolStripButtonPlay.Enabled = true;
                    toolStripButtonResumePlaying.Enabled = true;

                    if (firstMarked == true)
                    {
                        toolStripButtonMovetoStart.Enabled = true;
                        toolStripButtonMarkEnd.Enabled = true;
                    }

                    if (firstMarked && secondMarked)
                    {
                        toolStripButtonPreviewSnippet.Enabled = true;
                        toolStripButtonClearMarks.Enabled = true;
                        toolStripButtonSaveSnippet.Enabled = true;
                    }


                    toolStripButtonPause.Enabled = false;

                    if (firstMarked == true)
                    {
                        toolStripButtonMarkEnd.Enabled = true;
                    }                    
                }
                else if(playing == false && wc.CurrentTime != wc.TotalTime)
                {
                    playing = true;

                    toolStripButtonPause.Enabled = true;

                    toolStripButtonPlay.Enabled = false;
                    toolStripButtonResumePlaying.Enabled = false;
                    toolStripButtonMarkStart.Enabled = false;
                    toolStripButtonMarkEnd.Enabled = false;
                    toolStripButtonMovetoStart.Enabled = false;
                    timerPreviewSnippet.Enabled = false;
                    toolStripButtonClearMarks.Enabled = false;
                    toolStripButtonSaveSnippet.Enabled = false;

                    timerCheckPlayback.Start();

                    audioOutput.Play(); 
                }
            }
            else if ((e.KeyValue == 37 || e.KeyValue == 109 || e.KeyValue == 189) && fileLoaded == true)
            {
                //move to the left
                //MessageBox.Show("left");
                wc.CurrentTime = wc.CurrentTime - new TimeSpan(0, 0, 0, 0, 50);
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
            }
            else if ((e.KeyValue == 39 || e.KeyValue == 107 || e.KeyValue == 187) && fileLoaded == true)
            {
                //move to the right
                //MessageBox.Show("right");
                wc.CurrentTime = wc.CurrentTime + new TimeSpan(0, 0, 0, 0, 50);
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
            }
        }

        /***********************************************
         *handles the form load event
         **********************************************/
        private void snippetCreatorForm_Load(object sender, EventArgs e)
        {
            theConnectionString = "Provider=Microsoft.ACE.OLEDB.12.0;"
                           + @"data source=soundClips.accdb";   // Access 2010
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            OleDbCommand searchCommand;
            searchCommand = theConnection.CreateCommand();

            searchCommand.CommandText = String.Format("SELECT person FROM SoundClips ORDER BY person");
            OleDbDataReader reader = searchCommand.ExecuteReader();

            comboBoxPeople.Items.Add("Add person");

            // used for part 4.b
            if (reader.HasRows == true)
            {

                while (reader.Read())
                {
                    string person = reader.GetString(0);

                    if (comboBoxPeople.Items.Contains(person) == false)
                    {
                        comboBoxPeople.Items.Add(person);
                    }
                }
            }

            theConnection.Close();
        }

        /***********************************************
         *Handles when the selected index of the people combobox changes
         **********************************************/
        private void comboBoxPeople_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxPeople.SelectedIndex == 0)
            {
                AddPersonDialog child = new AddPersonDialog();
                child.ShowDialog();

                DialogResult result = child.DialogResult;

                if (result == DialogResult.OK)
                {
                    comboBoxPeople.Items.Add(child.personName);
                    comboBoxPeople.SelectedIndex = comboBoxPeople.Items.Count - 1;
                }
                else
                {
                    comboBoxPeople.SelectedIndex = -1;
                }
            }
        }

        /***********************************************
         *Handles the button to move the currentTime back
         **********************************************/
        private void toolStripButtonMoveBack_Click(object sender, EventArgs e)
        {
            if (fileLoaded == true)
            {
                wc.CurrentTime = wc.CurrentTime - new TimeSpan(0, 0, 0, 0, 50);
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
            }
        }

        /***********************************************
         *Handles the button to move the currentTime forward
         **********************************************/
        private void toolStripButtonMoveForward_Click(object sender, EventArgs e)
        {
            if (fileLoaded == true)
            {
                wc.CurrentTime = wc.CurrentTime + new TimeSpan(0, 0, 0, 0, 50);
                moveTrackBar(wc.CurrentTime, wc.TotalTime);
            }
        }

        /***********************************************
         *Handles the load tool strip menu item
         **********************************************/
        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Displays an OpenFileDialog so the user can select a Cursor.
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Filter = "WAV Files|*.wav";
            fileDialog.Title = "Select a WAV File";
            fileDialog.Multiselect = false;

            // Show the Dialog.
            // If the user clicked OK in the dialog and
            // a .WAV file was selected, open it.
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                if (fileLoaded == true)
                {
                    wfr.Close();
                    wc.Close();
                    audioOutput.Dispose();
                }                

                //load the new file
                if (File.Exists(loadedClip))
                {
                    File.Delete(loadedClip);
                }                

                //copy the file into the same directory as the .exe
                File.Copy(fileDialog.FileName, loadedClip);

                openNewFile(loadedClip);

                //converter
                if (wc.WaveFormat.Equals(CustomSoundClass.DefaultFormat) == false)
                {
                    WaitDialog waitDlg = new WaitDialog("Converting loaded file to the proper format");
                    waitDlg.Show();
                    this.Cursor = Cursors.WaitCursor;
                    try
                    {
                        audioOutput.Dispose();
                        wc.Close();
                        wfr.Close();

                        if (File.Exists("origFile.wav"))
                        {
                            File.Delete("origFile.wav");
                        }
                        File.Move(loadedClip, "origFile.wav");

                        WaveStream stream = new WaveFileReader("origFile.wav");

                        WaveFormatConversionStream str = new WaveFormatConversionStream(CustomSoundClass.DefaultFormat, stream);

                        WaveFileWriter.CreateWaveFile(loadedClip, str);

                        stream.Close();
                        str.Close();

                        File.Delete("origFile.wav");

                        openNewFile(loadedClip);

                        fileLoaded = true;
                        toolStripButtonPlay.Enabled = true;
                        toolStripButtonResumePlaying.Enabled = true;
                        toolStripButtonMarkStart.Enabled = true;
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                    finally
                    {
                        this.Cursor = Cursors.Arrow;
                        waitDlg.Close();                       
                    }
                }
                else
                {
                    fileLoaded = true;
                    toolStripButtonPlay.Enabled = true;
                    toolStripButtonResumePlaying.Enabled = true;
                    toolStripButtonMarkStart.Enabled = true;
                }
            }
        }
    }
}
